package comparador;

import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

@SuppressWarnings("serial")
public class Main extends JFrame{

	private File arc1;
	private File arc2;
	private File result;

	private JTextField campoArc1;
	private JTextField campoArc2;
	private JTextField campoArc3;

	private JButton botonBuscar;
	private JProgressBar barraProgreso;

	public Main() {
		campoArc1 = new JTextField(20);
		campoArc2 = new JTextField(20);
		campoArc3 = new JTextField(20);
		campoArc3.setEditable(false);

		setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		add(new PanelSelecArc("Seleccione el archivo con las palabras a buscar", campoArc1, 1));
		add(new PanelSelecArc("Seleccione el archivo donde buscar las palabras", campoArc2, 2));
		add(new PanelBotonBuscar());
		add(new PanelFicheroResultado());

		setTitle("Comparador de palabras");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pack();
		setResizable(false);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	private class PanelSelecArc extends JPanel {

		JButton botonAbrir;

		public PanelSelecArc(String mensaje, JTextField campo, int tipoArchivo) {

			botonAbrir = new JButton("Abrir");
			botonAbrir.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					JFileChooser selector = new JFileChooser();
					int opcion = selector.showOpenDialog(null);
					if (opcion == JFileChooser.APPROVE_OPTION) {
						if (tipoArchivo == 1) {
							arc1 = selector.getSelectedFile();
							campo.setText(arc1.getAbsolutePath());
						}
						else {
							arc2 = selector.getSelectedFile();
							campo.setText(arc2.getAbsolutePath());
						}
					}
				}
			});
			add(campo);
			add(botonAbrir);
			setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20),
					BorderFactory.createCompoundBorder(
							BorderFactory.createTitledBorder(mensaje),
							BorderFactory.createEmptyBorder(10, 10, 10, 10))));
		}
	}

	private class PanelBotonBuscar extends JPanel {

		public PanelBotonBuscar() {
			botonBuscar = new JButton("Buscar palabras");
			botonBuscar.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					
					BuscarPalabras buscar = new BuscarPalabras();
					buscar.start();
				}
			});
			barraProgreso = new JProgressBar();
			barraProgreso.setPreferredSize(new Dimension(400, 30));
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			JPanel pnBoton = new JPanel();
			pnBoton.add(botonBuscar);
			JPanel pnProgreso = new JPanel();
			pnProgreso.add(barraProgreso);
			add(pnBoton);
			add(pnProgreso);
			setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));
		}
	}

	private class PanelFicheroResultado extends JPanel {

		JButton abrirDire;

		public PanelFicheroResultado() {

			abrirDire = new JButton("Abrir Resultado");
			abrirDire.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					if (result == null)
						mensajePopUp("No hay archivo para mostrar");
					else
						try {
							Desktop.getDesktop().open(result);
						} catch (IOException e1) {
							mensajePopUp("No se pudo abrir: " + result.getAbsolutePath());
							e1.printStackTrace();
						}
					
				}
			});
			JPanel pnCampo = new JPanel();
			pnCampo.add(campoArc3);
			JPanel pnBoton = new JPanel();
			pnBoton.add(abrirDire);

			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			add(pnCampo);
			add(pnBoton);
			setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20),
					BorderFactory.createCompoundBorder(
							BorderFactory.createTitledBorder("Archivo de resultados se encuentra en esta ruta"),
							BorderFactory.createEmptyBorder(10, 10, 10, 10))));
		}
	}

	private class BuscarPalabras extends Thread {

		@Override
		public void run() {
			/*
			 * Leeremos las lineas de cada archivo de texto y separaremos
			 * las palabras individualmente en sendos Vector.
			 */

			Vector<Palabra> palabras = new Vector<Palabra>(); //Palabras a buscar
			Vector<String> texto = new Vector<String>(); //Texto donde buscaremos

			try {
				BufferedReader br = new BufferedReader(new FileReader(arc1));
				String linea = br.readLine();
				while (linea != null) {
					//Separamos palabras de esta linea.
					//Suponemos que en el texto estan separadas por un espacio en blanco
					String[] separadas = linea.split(" ");
					//Ahora las añadimos al Vector
					for (String pal: separadas)
						palabras.add(new Palabra(pal));
					//Siguiente linea
					linea = br.readLine();
				}
				br.close();
			} catch (FileNotFoundException e) {
				mensajePopUp("No se encuentra archivo:\n" + arc1.getAbsolutePath());
				e.printStackTrace();
			} catch (IOException e) {
				mensajePopUp("Error de acceso a archivo:\n" + arc1.getAbsolutePath());
				e.printStackTrace();
			}

			/*
			 * Tenemos las Palabras.
			 * Ahora necesitamos las lineas del texto donde buscar
			 */

			try {
				BufferedReader br = new BufferedReader(new FileReader(arc2));
				String linea = br.readLine();
				while (linea != null) {
					//En el Vector de texto metemos lineas enteras
					texto.add(linea);
					//Siguiente linea
					linea = br.readLine();
				}
				br.close();
			} catch (FileNotFoundException e) {
				mensajePopUp("No se encuentra archivo:\n" + arc2.getAbsolutePath());
				e.printStackTrace();
			} catch (IOException e) {
				mensajePopUp("Error de acceso a archivo:\n" + arc2.getAbsolutePath());
				e.printStackTrace();
			}

			/*
			 * Tenemos palabras y texto.
			 * Ahora hay que empezar a comparar.
			 * Usamos una expresion regular.
			 */
			//Primero configuramos barra progreso según la cantidad de palabras a buscar
			barraProgreso.setMaximum(palabras.size());
			barraProgreso.setValue(0);

			for (Palabra pal: palabras) {
				//Creamos patrón para la expresion regular con Palabra actual
				String patron = " \\b" + pal.getPalabra() + "\\b";
				//En el bucle anidado, recorremos cada linea del texto y comparamos
				for (String linea: texto) {
					Pattern patr = Pattern.compile(patron);
					Matcher mt = patr.matcher(linea);
					if (mt.find())
						pal.setEncontrada(true);
				}
				//Ya hemos terminado con esta palabra, aumentamos progreso
				try {
					Thread.sleep(100); //Ligero delay
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				barraProgreso.setValue(barraProgreso.getValue() + 1);
			}
			
			/*
			 * Ya hemos comprobado palabras y sabemos cuales coinciden
			 * y cuales no.
			 * Crearemos un nuevo archivo con dichas palabras.
			 */
			try {
				result = new File("encontradas.txt");
				BufferedWriter bw = new BufferedWriter(new FileWriter(result));
				for (Palabra pal: palabras)
					bw.write(pal.toString());
				bw.close();
				campoArc3.setText(result.getAbsolutePath());
			} catch (IOException e) {
				mensajePopUp("No se pudo grabar archivo de palabras encontradas en: " + result.getAbsolutePath());
				e.printStackTrace();
			}
		}
	}

	private void mensajePopUp(String msj) {
		JOptionPane.showMessageDialog(null, msj, "Comparador", JOptionPane.WARNING_MESSAGE);
	}

	public static void main(String[] args) {

		try{

			JFrame.setDefaultLookAndFeelDecorated(true);
			/*for (Object obj: UIManager.getInstalledLookAndFeels()) {
				System.out.println(obj.toString());
			}*/
			//JDialog.setDefaultLookAndFeelDecorated(true);
			UIManager.setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
			//UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
			//UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			//UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				new Main();
			}
		});
	}

}
